/**************************************************************/
/*                                                            */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Variable table size definitions                            */
/**************************************************************/

#define MAX_NAME_LEN   31
#define MAX_DATA_LEN   95
#define MAX_VAR_COUNT  512

/* error return codes */
#define NOFILENAME		101
#define BADSTATUSHEADER	99

/* Special HTML variable names */
#define REPLAYCOUNTVAR "REPLAYCOUNT"
#define INPUTDEBUGFILE "&INPUTDEBUGFILE"

/**************************************************************/
/*                                                            */
/* Variable name and value array definition                   */
/*                                                            */
/* Input variables are saved in this array.  Other variables  */
/*  can be added as well.  To build the output HTML page,     */
/*  the file containing the HTML page is written to           */
/*  stdout, with variable names being substituted with the    */
/*  corresponding values.  Variables are recognized by        */
/*  looking for a double % sequence.                          */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Variable table structure definition                        */
/**************************************************************/

typedef struct {
   char VarName[MAX_NAME_LEN + 1];
   char VarData[MAX_DATA_LEN + 1];
} VARSTRUCT;

/**************************************************************/
/* Maximum characters on a line in the prototype file         */
/**************************************************************/

#define MAX_LINE_LEN   255

/**************************************************************/
/* Maximum number of characters from browser                  */
/**************************************************************/

#define MAX_INPUT_CHAR 8192

/* 3270 field types */
#define ALPHA_FIELD   0
#define NUMERIC_FIELD 1
#define CICS_FIELD    2

/* Definition for HTML variable name to save status file name */
#define STATFILE "CICSSTATUSFILE"

/**************************************************************/
/* Function prototype definitions                             */
/**************************************************************/

void strupper(char *cStr);
void strcvrt(char *cStr, char cOld, char cNew);
static int TwoHex2Int(char *pC);

/* Common subroutines to handle CGI processing */
void urlDecode(char *p);
void ExpandChar(char *VarData, char ch, char *NewValue);
void get_input_string(char *input_stream);
void parse_input(char *varname, VARSTRUCT *pVars);
void build_header();
void write_html_page(VARSTRUCT *pVars, char *filename);
void build_trailer();

/* Subroutines to handle the variable table */
void AddVar(const char *varname, const char *vardata, VARSTRUCT *pVars);
void UpdateVar(const char *varname, const char *vardata, VARSTRUCT *pVars);
void GetVarVal(const char *varname, char *vardata, VARSTRUCT *pVars);
const char * GetVarPtr(const char *varname, VARSTRUCT *pVars);

/* Subroutines to process a CICS COBOL COMMAREA */
void short_to_comm(const short s, char *result, const int len);
void int_to_comm(const long i, char *result, const int len);
short reverseBytes(short *var);
int reverseBytes4(int var);
void char_from_comm(char *VarData, const char *commarea, const long Varlen);
void from_comm(char *VarData, const char *commarea, const long Varlen);
int  comm_to_int(char *commarea, long Varlen);
long comm_to_long(char *commarea, long Varlen);
void char_to_comm(const char *VarData, char *CommField, int maxLen, int isNumber);
void to_comm(const char *VarName, char *CommField, VARSTRUCT *pVars, int maxLen, int isNumber);
void toBMSfield(VARSTRUCT *pVars, char *name, char *BMSfield, short *BMSlen, 
				  short     *MAPlen, int maxLen,int isNumber);
void toBMSifdiff(VARSTRUCT *pVars, char *name, char *oldname, char *BMSfield,
                   short     *BMSlen, short *MAPlen, int maxLen, int isNumber);
void findDebugFile(char *input_stream, char *dbgname);

/* Subroutines to handle the CICS status file */
int      updateStatusfile(VARSTRUCT *vars,
						  const int replay,
                          const int tctualen,
                          const char* tctua,
                          const int calen,
                          const char* ca);

int      getStatusfileData(VARSTRUCT *vars,
                           int* replay,
                           int* tctualen,
                           char* tctua,
                           int* calen,
                           char* ca,
						   char* cics_user, 
						   char* cics_pw,
						   char* cics_sys,
						   char* cics_client);

/****************/
/* End of File  */
/****************/