/**************************************************************/
/*                                                            */
/*  CGI program for access to a modified version of one       */
/*   of the IVP programs.  Records can be displayed,          */
/*   added and updated.  This program calls a CICS program    */
/*   called NEWCALL, passing a COMMAREA, using the CICS ECI.  */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/*  This program demonstrates basic techniques for a proof    */
/*   of concept for CICS access from the Internet.            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required include for C functions                           */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/**************************************************************/
/* Include for common subroutines                             */
/**************************************************************/

#include "comsubs.h"
#include "ecimsg.h"

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#define CICS_W32
#include <cics_eci.h>

/**************************************************************/
/* Definitions for Standard header and BMS map areas used     */
/* in COMMAREAs                                               */
/**************************************************************/
#include "stdhead.h"
#include "dfhcga.h"
#include "dfhcgb.h"
#include "dfhcgc.h"

/**************************************************************/
/* COMMAREA definition                                        */
/**************************************************************/

#include "credcomm.h"

/* request types */
#define REQ_TYPE_CREATE   '0'
#define REQ_TYPE_UPDATE   '1'
#define REQ_TYPE_DISP     '2'
#define REQ_TYPE_BROWSE   '3'
#define REQ_TYPE_CLEAR    '4'
#define REQ_TYPE_FORWARD  '5'
#define REQ_TYPE_BACKWARD '6'

/* name of the HTML prototype file */
#define HTML_INQY_FILE   "credinqy.htm"
#define HTML_BROWSE_FILE "credbrws.htm"

/* Name of CICS programs to execute */
#define INQYPGM "NEWCALL "
#define BRWSPGM "NEWCBRW "

/* User transaction ids - these are checked by the CICS program */
#define ADDTRAN    "ADDS"
#define UPDATETRAN "UPDT"
#define BROWSETRAN "BRWS"

/* CICS BMS map names */
#define DETAILMAP "DETAIL  "
#define BROWSEMAP "BROWSE  "
#define MENUMAP   "MENU    "

/* Change button definition to allow conditional appearance of option */
#define CHGBUTTON "<input type=\"submit\" name=\"SUBMIT\" value=\"Change Account\">"

/**************************************************************/
/* Routine to clear any detail variables, in case of an       */
/*  ECI error                                                 */
/**************************************************************/

void clearVars(VARSTRUCT *pVars)

{
	UpdateVar("ACCTNO", "", pVars);
	UpdateVar("NAME", "", pVars);
	UpdateVar("ADDRESS", "", pVars);
	UpdateVar("PHONE", "", pVars);
	UpdateVar("DATE", "", pVars);
	UpdateVar("AMOUNT", "", pVars);
	UpdateVar("COMMENT", "", pVars);
	UpdateVar("OLDACCTNO", "", pVars);
	UpdateVar("NUMBER1", "", pVars);
	UpdateVar("NAME1", "", pVars);
	UpdateVar("AMOUNT1", "", pVars);
	UpdateVar("NUMBER2", "", pVars);
	UpdateVar("NAME2", "", pVars);
	UpdateVar("AMOUNT2", "", pVars);
	UpdateVar("NUMBER3", "", pVars);
	UpdateVar("NAME3", "", pVars);
	UpdateVar("AMOUNT3", "", pVars);
	UpdateVar("NUMBER4", "", pVars);
	UpdateVar("NAME4", "", pVars);
	UpdateVar("AMOUNT4", "", pVars);
}

/**************************************************************/
/* Routine to update the variable table with new values       */
/*  from the CICS commarea.                                   */
/**************************************************************/

void update_vars(VARSTRUCT *pVars, PCOMMAREA pCommArea)

{
    char   VarData[MAX_DATA_LEN + 1];

    if (memcmp(pCommArea->inqy.comm_head.MAPNAME,"DETAIL  ",
               sizeof(pCommArea->inqy.comm_head.MAPNAME)) == 0)
    {
       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.numbi,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.numbi));
       UpdateVar("ACCTNO", VarData, pVars);
       UpdateVar("OLDACCTNO", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.namei,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.namei));
       UpdateVar("NAME", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.addri,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.addri));
       UpdateVar("ADDRESS", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.phonei,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.phonei));
       UpdateVar("PHONE", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.datei,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.datei));
       UpdateVar("DATE", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.amounti,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.amounti));
       UpdateVar("AMOUNT", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbi.commenti,
                 sizeof(pCommArea->inqy.detail.dfhdgbi.commenti));
       UpdateVar("COMMENT", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbo.msg1o,
                 sizeof(pCommArea->inqy.detail.dfhdgbo.msg1o));
       UpdateVar("ERRMSG", VarData, pVars);

       from_comm(VarData, pCommArea->inqy.detail.dfhdgbo.msg3o,
                 sizeof(pCommArea->inqy.detail.dfhdgbo.msg3o));
       UpdateVar("ERRMSG2", VarData, pVars);
    }

    if (memcmp(pCommArea->inqy.comm_head.MAPNAME,"MENU    ",
               sizeof(pCommArea->inqy.comm_head.MAPNAME)) == 0)
    {
       if (strcmp(pCommArea->inqy.comm_head.USRTRNID,"BRWS") == 0)
       {
          from_comm(VarData, pCommArea->brws.menu.dfhdgao.msgo,
                    sizeof(pCommArea->brws.menu.dfhdgao.msgo));
          UpdateVar("ERRMSG", VarData, pVars);
       }
       else
       {
          from_comm(VarData, pCommArea->inqy.menu.dfhdgao.msgo,
                    sizeof(pCommArea->inqy.menu.dfhdgao.msgo));
          UpdateVar("ERRMSG", VarData, pVars);
       }
    } /* endif */

    if (memcmp(pCommArea->brws.comm_head.MAPNAME,"BROWSE  ",
               sizeof(pCommArea->brws.comm_head.MAPNAME)) == 0)
    {
       from_comm(VarData, pCommArea->brws.browse.dfhdgco.number1o,
                 sizeof(pCommArea->brws.browse.dfhdgco.number1o));
       UpdateVar("NUMBER1", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.name1o,
                 sizeof(pCommArea->brws.browse.dfhdgco.name1o));
       UpdateVar("NAME1", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.amount1o,
                 sizeof(pCommArea->brws.browse.dfhdgco.amount1o));
       UpdateVar("AMOUNT1", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.number2o,
                 sizeof(pCommArea->brws.browse.dfhdgco.number2o));
       UpdateVar("NUMBER2", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.name2o,
                 sizeof(pCommArea->brws.browse.dfhdgco.name2o));
       UpdateVar("NAME2", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.amount2o,
                 sizeof(pCommArea->brws.browse.dfhdgco.amount2o));
       UpdateVar("AMOUNT2", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.number3o,
                 sizeof(pCommArea->brws.browse.dfhdgco.number3o));
       UpdateVar("NUMBER3", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.name3o,
                 sizeof(pCommArea->brws.browse.dfhdgco.name3o));
       UpdateVar("NAME3", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.amount3o,
                 sizeof(pCommArea->brws.browse.dfhdgco.amount3o));
       UpdateVar("AMOUNT3", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.number4o,
                 sizeof(pCommArea->brws.browse.dfhdgco.number4o));
       UpdateVar("NUMBER4", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.name4o,
                 sizeof(pCommArea->brws.browse.dfhdgco.name4o));
       UpdateVar("NAME4", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.amount4o,
                 sizeof(pCommArea->brws.browse.dfhdgco.amount4o));
       UpdateVar("AMOUNT4", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.msg2o,
                 sizeof(pCommArea->brws.browse.dfhdgco.msg2o));
       UpdateVar("ERRMSG", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.msg0o,
                 sizeof(pCommArea->brws.browse.dfhdgco.msg0o));
       UpdateVar("ERRMSG2", VarData, pVars);

       from_comm(VarData, pCommArea->brws.browse.dfhdgco.msg1o,
                 sizeof(pCommArea->brws.browse.dfhdgco.msg1o));
       UpdateVar("ERRMSG3", VarData, pVars);
    }

    if (memcmp(pCommArea->inqy.comm_head.MAPNAME,"SENDTEXT",
               sizeof(pCommArea->inqy.comm_head.MAPNAME)) == 0)
    {
       from_comm(VarData, pCommArea->inqy.comm_head.textarea,
                 sizeof(pCommArea->inqy.comm_head.textarea));
       UpdateVar("ERRMSG", VarData, pVars);
    } /* endif */
}

/**************************************************************/
/* Routine to build the COMMAREA for the second pass, for     */
/*  adds and updates.                                         */
/**************************************************************/

int	build_detail(VARSTRUCT *pVars, PCOMMAREA pCommArea)

{
	short	maplen=0;

	toBMSfield(pVars, "ACCTNO",
			   pCommArea->inqy.detail.dfhdgbi.numbi,
			   &pCommArea->inqy.detail.dfhdgbi.numbl,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.numbi),
			   NUMERIC_FIELD);

	toBMSfield(pVars, "NAME",
			   pCommArea->inqy.detail.dfhdgbi.namei,
			   &pCommArea->inqy.detail.dfhdgbi.namel,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.namei),
			   ALPHA_FIELD);

	toBMSfield(pVars, "ADDRESS",
			   pCommArea->inqy.detail.dfhdgbi.addri,
			   &pCommArea->inqy.detail.dfhdgbi.addrl,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.addri),
			   ALPHA_FIELD);

	toBMSfield(pVars, "PHONE",
			   pCommArea->inqy.detail.dfhdgbi.phonei,
			   &pCommArea->inqy.detail.dfhdgbi.phonel,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.phonei),
			   ALPHA_FIELD);

	toBMSfield(pVars, "DATE",
			   pCommArea->inqy.detail.dfhdgbi.datei,
			   &pCommArea->inqy.detail.dfhdgbi.datel,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.datei),
			   ALPHA_FIELD);

	toBMSfield(pVars, "AMOUNT",
			   pCommArea->inqy.detail.dfhdgbi.amounti,
			   &pCommArea->inqy.detail.dfhdgbi.amountl,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.amounti),
			   ALPHA_FIELD);

	toBMSfield(pVars, "COMMENT",
			   pCommArea->inqy.detail.dfhdgbi.commenti,
			   &pCommArea->inqy.detail.dfhdgbi.commentl,
			   &maplen,
			   sizeof(pCommArea->inqy.detail.dfhdgbi.commenti),
			   ALPHA_FIELD);

	/* return the total length of all fields */
	return maplen;
}

/**************************************************************/
/* Routine to identify a variable and copy the data           */
/*  into the commarea.                                        */
/**************************************************************/

void build_commarea(VARSTRUCT *pVars,
                    PCOMMAREA pCommArea,
					int  replay,
					int  calen,
					char *ca,
                    char *reqType,
					char *cics_client)

{
	char	vardata[MAX_DATA_LEN + 1];
	char	account[MAX_DATA_LEN + 1];
	char	oldacct[MAX_DATA_LEN + 1];
	char	*acct;
	char	*tran="";
	char	*mapname="";
	short	maplen=0;
	char	direction;
	int		pagecounter=0;
	const char *countvar;

	/* Get the value of the submit button */
    GetVarVal("SUBMIT", vardata, pVars);

	/* initialize the request type */
	*reqType = 0;

	if (strcmp(vardata, "Add New Account") == 0)
	{
		*reqType = REQ_TYPE_CREATE;

		/* Indicate which account number to use */
		acct = "ACCTNO";
	}

	if (strcmp(vardata, "Change Account") == 0)
	{
		*reqType = REQ_TYPE_UPDATE;

		/* set the transaction code */
		tran = UPDATETRAN;

		/* Indicate which account number to use */
		acct = "ACCTNO";

		/* Get the account number                    */
		GetVarVal("ACCTNO", account, pVars);

		/* Get the old account hidden variable       */
		GetVarVal("OLDACCTNO", oldacct, pVars);

		/* check if they are the same           */
		/* if they are, then we can update the  */
		/* record.  Also, make sure that the    */
		/* user COMMAREA exists, which we need  */
		/* for the second pass.  If not, then   */
		/* we will convert the request into a   */
		/* display request.                     */
		if ((strcmp(account, oldacct) != 0) || (calen == 0))
		{
			/* Account number has been changed */
			/* Treat this as an inquiry.       */
			*reqType = REQ_TYPE_DISP;
		}
	}


    if (strcmp(vardata, "Show Details") == 0)
	{
		*reqType = REQ_TYPE_DISP;

		/* Indicate which account number to use */
		acct = "ACCTNO";
	}

    if (strcmp(vardata, "Browse Accounts") == 0)
       *reqType = REQ_TYPE_BROWSE;

    if (strcmp(vardata, "Clear Form") == 0)
       *reqType = REQ_TYPE_CLEAR;

    if (strcmp(vardata, "Page Forward") == 0)
	{
		*reqType = REQ_TYPE_FORWARD;
		direction = 'F';
	}

    if (strcmp(vardata, "Page Backward") == 0)
	{
		*reqType = REQ_TYPE_BACKWARD;
		direction = 'B';
	}

	if (strcmp(vardata, "Display Account 1") == 0)
	{
		*reqType = REQ_TYPE_DISP;
		acct = "NUMBER1";
	}

    if (strcmp(vardata, "Display Account 2") == 0)
	{
		*reqType = REQ_TYPE_DISP;
		acct = "NUMBER2";
	}

    if (strcmp(vardata, "Display Account 3") == 0)
	{
		*reqType = REQ_TYPE_DISP;
		acct = "NUMBER3";
	}

    if (strcmp(vardata, "Display Account 4") == 0)
	{
		*reqType = REQ_TYPE_DISP;
		acct = "NUMBER4";
	}

    if (strcmp(vardata, "Return to Inquiry") == 0)
	{
		*reqType = REQ_TYPE_DISP;
		acct = "ACCTNO";
	}

	/* Check for replays of previous pages      */
	/* These are not allowed for browse forward */
	/* or backward operations and changes.      */
	/* They are ignored for all other types of  */
	/* requests.                                */
	if ((*reqType == REQ_TYPE_FORWARD) || (*reqType == REQ_TYPE_BACKWARD) ||
		(*reqType == REQ_TYPE_UPDATE))
	{
		/* Check for replay */
		/* First, find the REPLAYCOUNT variable */
		/* from the HTML form.                  */
		countvar = GetVarPtr(REPLAYCOUNTVAR, pVars);
		if (countvar != NULL)
		{
			/* convert to an integer */
			pagecounter = atoi(countvar);

			/* Allow for the file counter being already updated */
			pagecounter++;
		}

		/* Check if the page matches the status file value */
		if (pagecounter != replay)
		{
			/* convert the request to a clear */
			*reqType = REQ_TYPE_CLEAR;

			/* Add an error message to the variable table */
			AddVar("ERRMSG", "Replays of previous pages not allowed", pVars);
		}
	}

    switch (*reqType)
    {
	/* To update a record requires two user      */
	/* interactions.  In the first one, the user */
	/* enters the account number and selects     */
	/* display.  The account information is      */
	/* displayed, and the oldaccount field is    */
	/* set as a hidden variable in the HTML      */
	/* page.  The user then changes the data and */
	/* selects update again.  This time, the     */
	/* fields which have been changed are        */
	/* updated in the file.  The first decision  */
	/* is to determine whether this is the first */
	/* interaction for this account number or    */
	/* second.                                   */
        case REQ_TYPE_UPDATE:

		   /* Set the COMMAREA fields */
		   maplen = build_detail(pVars, pCommArea);

		   /* Use the detail map */
		   mapname = DETAILMAP;

		   /* set the transaction code */
		   tran = UPDATETRAN;

           break;

        case REQ_TYPE_DISP:
        case REQ_TYPE_CREATE:
           /* This routine will read a record and       */
           /* display the data to the user.  It will    */
           /* drive the INQY transaction to get the     */
           /* data.  The INQY transaction will expect   */
           /* a record number in the keyi field in the  */
           /* menu map.  It uses the keyl field to see  */
           /* if data was entered in the field.         */

			/* Since the host transaction checks the    */
			/* length of the user COMMAREA, we will     */
			/* force the length to zero.                */
			calen = 0;

           /* move the account number to the keyi field */
           toBMSfield(pVars, acct,
                      pCommArea->inqy.menu.dfhdgai.keyi,
                      &pCommArea->inqy.menu.dfhdgai.keyl,
                      &maplen,
                      sizeof(pCommArea->inqy.menu.dfhdgai.keyi),
                      NUMERIC_FIELD);

			/* set the map name */
			mapname = MENUMAP;

			if (*reqType == REQ_TYPE_DISP)
			{
				/* set the transaction code */
				tran = UPDATETRAN;
			}
			else
			{
				/* set the transaction code */
				tran = ADDTRAN;
			}

           break;

        case REQ_TYPE_BROWSE:
           /* move the account number to the keyi field */
           toBMSfield(pVars, "ACCTNO",
                      pCommArea->brws.menu.dfhdgai.keyi,
                      &pCommArea->brws.menu.dfhdgai.keyl,
                      &maplen,
                      sizeof(pCommArea->brws.menu.dfhdgai.keyi),
                      NUMERIC_FIELD);

			/* Since the host transaction checks the    */
			/* length of the user COMMAREA, we will     */
			/* force the length to zero.                */
			calen = 0;

           /* set the transaction code */
           tran = BROWSETRAN;

           /* set the map name */
           mapname = MENUMAP;

           break;

        case REQ_TYPE_CLEAR:
           break;

        case REQ_TYPE_FORWARD:
        case REQ_TYPE_BACKWARD:
			/* move the direction to the BMS field */
			pCommArea->brws.browse.dfhdgci.diri = direction;

			/* Set the BMS length fields */
			pCommArea->brws.browse.dfhdgci.dirl = 1;
			maplen++;

			/* set the transaction code */
			tran = BROWSETRAN;

			/* set the map name */
			mapname = BROWSEMAP;

           break;

        default:
           break;
    } /* endswitch */

    /* set the fields in the standard header */
	setStandardHeader(&pCommArea->inqy.comm_head, calen, maplen, mapname, tran, cics_client);

	/* Move the user COMMAREA from the last transaction to the COMMAREA */
	if (calen > 0)
	{
		memcpy(&pCommArea->inqy.user_comm, ca, calen);
	}
}

void Display_Quit()

{
    build_header();
    printf("<h1>Thanks for using Credit Account Information</h1>\n");
    build_trailer();
}

/**************************************************************/
/* Routine to free all allocated storage                      */
/**************************************************************/

void free_all(PCOMMAREA pCommArea, VARSTRUCT *pVars)

{
	/* Be a good citizen and free the COMMAREA */
	if (pCommArea != NULL)
	{
		free(pCommArea);
	}

	/* and free the Variable table */
	if (pVars != NULL)
	{
		free(pVars);
	}
}

int main(int argc, char *argv[])

{
	char        *pgmName;
    PCOMMAREA   pCommArea;
    char        reqType;
	char		abendcode[ABCODE_LEN + 1];
    VARSTRUCT   *pVars;
    char        input_stream[MAX_INPUT_CHAR];
    char        *token;
    short       rc;
    char        cics_user[MAX_USERID_LEN + 1];
    char        cics_pw[MAX_PW_LEN + 1];
	char        cics_sys[MAX_SYS_NAME_LEN + 1];
	char		cics_client[MAX_CLIENT_NAME_LEN + 1];
    char        *dbginputname;              /* debug file name */
    char        dbgname[MAX_DATA_LEN + 1];  /* debug file name */
    char        *filename;
	char		*chgbutton = CHGBUTTON;		/* HTML value for change button */
	char		ca[32768];					/* User COMMAREA from last tran */
	char		tctua[256];					/* TCTUA */
	char		tempvar[MAX_DATA_LEN + 1];  /* RESETCOUNT work variable */ 
    FILE        *dbgfile;                   /* debug file */
	int			replay=0;					/* Replay counter */
	int			calen=0;					/* user COMMAREA length */
	int			tctualen=0;					/* TCTUA length */
	int			ecitoken=0;					/* For ECI extended calls */

/**************************************************************/
/*     Turn off buffering for stdout                          */
/**************************************************************/
    setvbuf(stdout,NULL,_IONBF,0);

/**************************************************************/
/* Start a new html page                                      */
/**************************************************************/
    printf("Content-type: text/html\n\n");
    fflush(NULL);

/**************************************************************/
/* Allocate a commarea and variable area                      */
/**************************************************************/
	pCommArea = malloc(sizeof(COMMAREA));
	pVars = (VARSTRUCT *)malloc(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1));
    if ((pCommArea == NULL) || (pVars == NULL))
	{
		build_header();
		printf("<h1>Error allocating storage</h1>");
		build_trailer();
		free_all(pCommArea, pVars);
		exit(1);
	}

/**************************************************************/
/* Initialize the commarea variables to default values        */
/**************************************************************/

    memset(pCommArea,'\0',sizeof(COMMAREA));
    memset(pVars,'\0',(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1)));
    memset(cics_sys, '\0', sizeof(cics_sys));
	memset(ca, 0, sizeof(ca));
	memset(tctua, 0, sizeof(tctua));

/**************************************************************/
/* Get the form input into variable input_stream              */
/**************************************************************/

    if (argc>1)        /* check for debugging */
    {
       dbginputname = argv[1];
       if (strcmp(dbginputname,"debug") == 0)
       {
          strcpy(input_stream,"&ACCTNO=12345&NAME=FOGHORN+T+LEGHORN&ADDRESS=1234+BURBANK");
          /*strcat(input_stream,"&SUBMIT=Add New Account");*/
          /*strcat(input_stream,"&SUBMIT=Change Account");*/
          /*strcat(input_stream,"&INPUTDEBUGFILE=debug.txt");*/
		  strcat(input_stream,"&CICSSTATUSFILE=c:\\CICSTEMP\\CICSCICSMROJIMM2");
          strcat(input_stream,"&SUBMIT=Show Details");
       }
       else
       {
          if ((dbgfile = fopen(dbginputname,"r")) != NULL)
          {
             fgets(input_stream, MAX_INPUT_CHAR, dbgfile);
             fclose(dbgfile);
          } /* endif */
       } /* endif */
    }
    else
    {
       get_input_string(input_stream);
    } /* endif */

/**************************************************************/
/* Check if we want to write out the input for debuging       */
/* purposes.  To easily capture the input form variables,     */
/* set a variable named INPUTDEBUGFILE to the name of         */
/* the file to be written out with the input from the         */
/* HTML form.  This input can then be fed back into           */
/* the program as a test case.  This allows easy debugging    */
/* of the CGI program outside of a web server.                */
/**************************************************************/

    /* find the debug file name in the input stream */
    /* and remove it from the input stream.         */
    findDebugFile(input_stream, dbgname);
    if (strlen(dbgname) != 0)
    {
		/* Expand escape sequences to proper characters */
		urlDecode(dbgname);

       if ((dbgfile = fopen(dbgname, "w")) != NULL)
       {
          fprintf(dbgfile,"%s",input_stream);
          fclose(dbgfile);
       } /* endif */
    } /* endif */

/**************************************************************/
/* Parse the input into fields                                */
/**************************************************************/

    /* get the first variable                                 */
    token = strtok(input_stream, "&");
    while( token != NULL )
     {
      /* Expand escape sequences to proper characters         */
      urlDecode(token);

      /* Process the variable                                 */
      parse_input(token, pVars);

      /* Get the next variable                                */
      token = strtok(NULL, "&");
     }

	/**************************************************/
	/* Read the CICS transaction STATUS file.         */
	/* This will retrieve any saved COMMAREA from the */
	/* previous transaction, as well as any TCTUA.    */
	/* These areas will be passed back to the next    */
	/* CICS transaction.                              */
	/**************************************************/

	rc = getStatusfileData(pVars, &replay, &tctualen, tctua, &calen, ca, 
						   cics_user, cics_pw, cics_sys, cics_client); 

	/* check if it worked */
	if (rc == 0)
	{
		/**************************************************************/
		/* Build the CommArea                                         */
		/**************************************************************/

		build_commarea(pVars, pCommArea, replay, calen, ca, &reqType, cics_client);
	}
	else
	{
		/* Didn't work - set an error message and */
		/* force the request type to clear.       */
		reqType = REQ_TYPE_CLEAR;
		AddVar("ERRMSG", "Unable to read status file", pVars);
	}

    /* Check for invalid request type */
    if (reqType == '\0')
     {
		Display_Quit();
		free_all(pCommArea, pVars);
		exit(0);
     }

/**************************************************************/
/* Get the CICS program name                                  */
/**************************************************************/

    if ((reqType == REQ_TYPE_BROWSE)   ||
        (reqType == REQ_TYPE_FORWARD)  ||
        (reqType == REQ_TYPE_BACKWARD))
    {
       pgmName = BRWSPGM;
    }
    else
    {
       pgmName = INQYPGM;
    } /* endif */

/**************************************************************/
/* Now issue the ECI call                                     */
/**************************************************************/

    if (reqType == REQ_TYPE_CLEAR)
    {
		rc = ECI_NO_ERROR;

		/* Clear the detail variables, so we don't */
		/* redisplay old data                      */
		clearVars(pVars);

		/* No user COMMAREA */
		calen = 0;
	}
    else
	{
		if (reqType == REQ_TYPE_CREATE)
		{
			/* Perform an extended call, to reduce the host */
			/* overhead.                                    */
			rc = perfECIext(pgmName, cics_user, cics_pw, cics_sys,
							(char *) pCommArea, sizeof(COMMAREA), pVars, abendcode, 
							&ecitoken);
		}
		else
		{
			rc = performECI(pgmName, cics_user, cics_pw, cics_sys,
							(char *) pCommArea, sizeof(COMMAREA), pVars, abendcode, 
							&ecitoken);
		}

		/**************************************************************/
		/* Check the results                                          */
		/**************************************************************/


		if (rc == ECI_NO_ERROR)
		{
			/* Get the current user COMMAREA length */
			calen = comm_to_int(pCommArea->inqy.comm_head.USRCALEN, 
								sizeof(pCommArea->inqy.comm_head.USRCALEN));

			/* Check if we need to make a second call */
			/* This is only required for update or    */
			/* add transactions.                      */
			if (reqType == REQ_TYPE_CREATE)
			{
				/* Second transaction is needed */
				/* Do we have what we need for the second transaction */
				/* to be run?                                         */
				if (calen > 0)
				{
					/* Update the COMMAREA for the second pass */
					build_detail(pVars, pCommArea);

					/* Set the user transaction name */
					char_to_comm(ADDTRAN, pCommArea->inqy.comm_head.USRTRNID,
								 sizeof(pCommArea->inqy.comm_head.USRTRNID), ALPHA_FIELD);

					/* Perform the actual update or add */
					rc = performECI(pgmName, cics_user, cics_pw, cics_sys,
									(char *) pCommArea, sizeof(COMMAREA), pVars, abendcode, 
									&ecitoken);
				}
				else
				{
					/* must have had a problem - just drop the */
					/* extended call.                          */
					rc = perfCommit(pgmName, cics_user, cics_pw, cics_sys,
									(char *) pCommArea, sizeof(COMMAREA), pVars, abendcode, 
									&ecitoken);
				}
			}

			/**************************************************************/
			/* Update the variables with new values from CICS.            */
			/**************************************************************/
			update_vars(pVars, pCommArea);

			/* Check for successful display */
			if ((reqType = REQ_TYPE_DISP) && (calen > 0))
			{
				/* Set the Change button variable so that a */
				/* change button will be displayed          */
				AddVar("CHANGEBUTTON", chgbutton, pVars);
			}
		}
		else
		{
			/* Clear the detail variables, so we don't */
			/* redisplay old data                      */
			clearVars(pVars);

			/* Set the user COMMAREA length to 0 */
			calen = 0;

			/* Create a user error message */
			UpdateVar("ERRMSG", "ERROR CALLING CICS", pVars);
		}
	}

	/* Update the COMMAREA length */
	calen = comm_to_int(pCommArea->inqy.comm_head.USRCALEN, 
						sizeof(pCommArea->inqy.comm_head.USRCALEN));

	/* Check if there is a User COMMAREA */
	if (calen > 0)
	{
		/* Save the user COMMAREA */
		memcpy(ca, &pCommArea->inqy.user_comm, calen);
	}

	/* Check if there is a TCTUA */
	if (tctualen > 0)
	{
		/* Save the TCTUA */
		/* In this example, there is no TCTUA, so */
		/* there is nothing to save               */
	}

	/* Update status file with a new COMMAREA and TCTUA. */
	/* The replay counter will also be incremented.      */
	rc = updateStatusfile(pVars, replay, tctualen, tctua, calen, ca);

	/* check if it worked */
	if (rc != 0)
	{
		AddVar("ERRMSG2", "Unable to update status file", pVars);
	}

/**************************************************************/
/* Build the new HTML page                                    */
/**************************************************************/

    if (memcmp(pCommArea->inqy.comm_head.MAPNAME, "BROWSE", 6) == 0)
    {
       filename = HTML_BROWSE_FILE;
    }
    else
    {
       filename = HTML_INQY_FILE;
    } /* endif */

	/* Make sure that the REPLAYCOUNT variable matches the current file */
	/* This prevents replays from getting the file and the HTML form    */
	/* from getting out of sync.                                        */
	itoa(replay, tempvar, 10);			
	UpdateVar(REPLAYCOUNTVAR, tempvar, pVars);

	/* Write out the output HTML page */
    write_html_page(pVars, filename);

/**************************************************************/
/* Finished - free the acquired storage and return normally   */
/**************************************************************/

	free_all(pCommArea, pVars);
    return(0);
}

/****************/
/* End of File  */
/****************/