// Comsubs.cpp: implementation of the CComsubs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Comsubs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define TRACE_FILE_NAME "C:\\TEMP\\CICSISA.TRC"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CComsubs::CComsubs()
{

}

CComsubs::~CComsubs()
{

}

////////////////////////////////////////////////////////////////
// Routine to get the value of a variable (CString) and then  
//  copy it into the CICS commarea.                           
////////////////////////////////////////////////////////////////

void CComsubs::to_comm(CString & Vardata, char * CommField, int maxLen, int isNumber)

{
    int     len;
    LPTSTR  userdata;

	// Make sure we don't have a null pointer
	ISAPIASSERT(CommField != NULL);

	// Make sure that the length is > 0
	ISAPIASSERT(maxLen > 0);

    // Get the length of the CString data 
    len = Vardata.GetLength();

    // Get a pointer to the CString data 
    userdata = Vardata.GetBuffer(0);

    if (len > maxLen)
     {
       len = maxLen;
     }

    // set the field to blanks or zeros 
    if (isNumber == NUMERIC_FIELD)
    {
       // numeric field - initialize to zeros 
       memset(CommField,'0',maxLen);

       // copy the data, if there is any, right justified 
       if (len > 0)
       {
          memcpy(CommField + maxLen - len, userdata, len);
       }
    }
    else
    {
       // alpha field - initialize to blanks 
       if (isNumber == ALPHA_FIELD)
       {
          memset(CommField,' ',maxLen);
       }
       else
       {
          memset(CommField,'\0',maxLen);
       }

       // copy the data, if there is any, left justified 
       if (len > 0)
       {
          memcpy(CommField, userdata, len);
       }
    }

    // Release the buffer pointer 
    Vardata.ReleaseBuffer();
}

////////////////////////////////////////////////////////////////
// Routine to move data into a BMS map type of field in the
// CICS COMMAREA.  This routine uses the to_comm routine
// above to move the data into the CICS COMMAREA, and then
// sets the BMS length field to the length of the input
// data.  This length is also added to a total length field
// that is passed in as the MAPlen parameter.
////////////////////////////////////////////////////////////////

void CComsubs::toBMSfield(CString& Vardata, char * BMSfield, short * BMSlen, short * MAPlen, int maxLen, int isNumber)

{
   int    len;

   to_comm(Vardata, BMSfield, maxLen, isNumber);

   // get the length of the new data 
   len = Vardata.GetLength();

   // make sure it is not too long 
   if (len > maxLen)
   {
      len = maxLen;
   }

   // set the BMS map length field 
   *BMSlen = len;

   // set the total BMS bytes in map field                   
   *MAPlen += len;
}

////////////////////////////////////////////////////////////////
// Routine to move data into a BMS like field if the new
// data is different from the previous data.
////////////////////////////////////////////////////////////////

void CComsubs::toBMSifdiff(CString& Vardata, CString& olddata, char* BMSfield, short* BMSlen, short* MAPlen, int maxLen, int isNumber)

{
	if (Vardata.Compare(olddata) != 0)
	{
		toBMSfield(Vardata, BMSfield, BMSlen, MAPlen, maxLen, isNumber);
	}
}

////////////////////////////////////////////////////////////////
// Routine to take data out of the CICS commarea and turn     
//  it into a valid CString, stripping trailing blanks.       
////////////////////////////////////////////////////////////////

void CComsubs::from_comm(CString & submit, const char * commarea, long Varlen)

{
   int  len;
   char temp[4096];

   // Check some of the input parameters
   ISAPIASSERT(commarea != NULL);
   ISAPIASSERT(Varlen > 0);

   // Erase the current contents of the CString variable 
   submit.Empty();

   // Make sure we don't overflow our work variable 
   len = Varlen;
   if (len > 4095)
   {
      len = 4095;
   } // endif 

   // Get the data into a work variable 
   memcpy(temp, commarea, len);
   temp[len] = 0;

   // strip trailing blanks 
   len--;
   while (temp[len] == ' ')
    {
      temp[len] = '\0';
      len--;

      if (len < 0)
         break;
    } // endwhile 

   submit = temp;
}

////////////////////////////////////////////////////////////////
// Routine to save an integer as a number string              
////////////////////////////////////////////////////////////////

void CComsubs::int_to_comm(const long i, char * result, const int len)

{
   char temp[18];
   int maxlen;

   // Check input parameters
   ISAPIASSERT(result != NULL);
   ISAPIASSERT(len > 0);

   itoa(i, temp, 10);

   if (len < 12)
   {
      maxlen = len;
   }
   else
   {
      maxlen = 11;
   } // endif 

   if (maxlen > 0)
   {
      memset(result, '0', maxlen);
      if (strlen(temp) > 0)
      {
         memcpy(result + maxlen - strlen(temp), temp, strlen(temp));
      } // endif 
   } // endif 
}

////////////////////////////////////////////////////////////////
// Routine to extract an integer from a COMMAREA character field              
////////////////////////////////////////////////////////////////

long CComsubs::comm_to_int(const char* field, const int len)

{
	CString s;

	// Initialize the string to a null
	s.Empty();

	/* Make sure the length is reasonable */
	if (len < 255)
	{
		/* Extract the number */
		from_comm(s, field, len);
	}

	/* Convert to an integer and return the integer value  */
	return atol(s);
}

//////////////////////////////////////////////////////
//
// This routine is to be used only during development 
// of the extension, when it is being executed in a 
// debug environment.  This code is not thread safe
// and should not be used in a production environment.
// This code should only compile when compiled in the
// DEBUG configuration, and should not create any output
// when compiled in a production environment.  This should
// allow any tracing statements to be left in the program.
//
//////////////////////////////////////////////////////

void CComsubs::writeTraceLine(const char* traceData)

{
#ifdef _DEBUG
	FILE	*tracefile;
	int		year;
	char	currtime[20];	// work variable for ASCII time
	char	currdate[20];	// work variable for ASCII date
	SYSTEMTIME	curtime;	// Work variable for current time

	// open the trace file for append
	tracefile = fopen(TRACE_FILE_NAME, "a");

	if (tracefile != NULL)
	{
		// Get the current time and date
		GetLocalTime(&curtime);

		// Format the date and and time
		year = curtime.wYear%100;
		sprintf(currdate, "%2.2d/%2.2d/%2.2d", year, curtime.wMonth, curtime.wDay);
		sprintf(currtime, "%2.2d:%2.2d:%2.2d.%3.3d", curtime.wHour, curtime.wMinute, 
				curtime.wSecond, curtime.wMilliseconds);

		fprintf(tracefile, "%s %s %s\n", currdate,currtime, traceData);
		fclose(tracefile);
	}
#endif
}

//////////////////////////////////////////////////////////
//
// Subroutine to turn a 4-bit number into a hex character
//
//////////////////////////////////////////////////////////

void data2hex(char *ch)

{
   if (((*ch) >= 0) && ((*ch) <= 9))
   {
      (*ch) += 48;  // turn into number 0-9 
   }
   else
   {
      (*ch) += 55;  // turn into letter A-F 
   } // endif 
}

//////////////////////////////////////////////////////
//
// This routine is to be used only during development 
// of the extension, when it is being executed in a 
// debug environment.  This code is not thread safe
// and should not be used in a production environment.
// This code should only compile when compiled in the
// DEBUG configuration, and should not create any output
// when compiled in a production environment.  This should
// allow any tracing statements to be left in the program.
//
//////////////////////////////////////////////////////

void CComsubs::dumpArea(const char* traceData, const int len)

{
#ifdef _DEBUG
	char*	envvar;			// pointer to trace file name environment variable
	FILE	*tracefile;
	int		year;
	char	currtime[20];	// work variable for ASCII time
	char	currdate[20];	// work variable for ASCII date
	SYSTEMTIME	curtime;	// Work variable for current time
	char	hexStr[41];
	char	asciiStr[41];
	char	ch;
	int		i;
	int		j;
	int		k;
	int		buffer=0;
	int		remaining;
	int		offset;
	int		count;

	// keep track of how much we have to do 
	remaining = len;

	// check if the CICSISATRACE environment variable is set
	envvar = getenv("CICSISATRACE");

	// Is the trace file name set in the environment?
	if (envvar == NULL)
	{
		// Use the default file name
		envvar = TRACE_FILE_NAME;
	}

	// open the trace file for append
	tracefile = fopen(envvar, "a");

	if (tracefile != NULL)
	{
		// Get the current time and date
		GetLocalTime(&curtime);

		// Format the date and and time
		year = curtime.wYear%100;
		sprintf(currdate, "%2.2d/%2.2d/%2.2d", year, curtime.wMonth, curtime.wDay);
		sprintf(currtime, "%2.2d:%2.2d:%2.2d.%3.3d", curtime.wHour, curtime.wMinute, 
				curtime.wSecond, curtime.wMilliseconds);
		fprintf(tracefile, "%s %s Data at location %8.8X length %d\n", 
				currdate,currtime,traceData,len);

		while (remaining > 0)
		{
			if (remaining > 16)
			{
				count = 16;
			}
			else
			{
				count = remaining;
			} // endif 

			// initialize the output strings and buffer offset 
			offset = buffer;
			i = 0;    // initialize variable offset 
			j = 0;    // initialize variable offset 
			k = 0;    // initialize counter 

			while (k < count)
			{
				ch = (unsigned char) traceData[buffer] >> 4;
				data2hex(&ch);
				hexStr[i++] = ch;

				ch = (unsigned char) traceData[buffer] & 0x0F;
				data2hex(&ch);
				hexStr[i++] = ch;

				if ((traceData[buffer] > ' ') &&
					((unsigned char) traceData[buffer] <= 127))
				{
					asciiStr[j++] = traceData[buffer];
				}
				else
				{
					asciiStr[j++] = '.';
				} // endif 

				// Go on to the next character 
				remaining--;
				buffer++;
				k++;   // increment counter 

				// Insert a blank for every fourth character 
				if (k%4 == 0)
				{
					hexStr[i++] = ' ';
				} // endif 
			} // endwhile 

			hexStr[i]   = '\0'; // Terminate the string 
			asciiStr[j] = '\0'; // Terminate the string 

			if (count > 0)
			{
				fprintf(tracefile, "%.4X %-36s  %s\n",
						offset,hexStr,asciiStr);
			}
		} // endwhile 
		fclose(tracefile);
	}
#endif
}