// Inqycomm.cpp : implementation file
//

#include "stdafx.h"
#include "Inqycomm.h"

// include for common COMMAREA subroutines
#include "comsubs.h"

// include for ECI subroutines
#include "ecisubs.h"

// Defines for user COMMAREA lengths
#define INQUIRECOMMSIZE 0
#define STARTBRCOMMSIZE 0
#define BRWSPGM			"NEWCBRW "
#define BROWSEMAPNAME	"BROWSE"
#define INQYPGM			"NEWCALL "
#define MENUMAPNAME		"MENU"
#define DETAILMAPNAME	"DETAIL"
#define INQUIRETRAN		"UPDT"
#define CHANGETRAN		"UPDT"
#define ADDTRAN			"ADDS"
#define BROWSETRAN		"BRWS"

// Change button definition to allow conditional appearance of option
#define CHGBUTTON "<input type=\"submit\" name=\"SUBMIT\" value=\"Change Account\">"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CInqycomm

//IMPLEMENT_DYNCREATE(CInqycomm, CCmdTarget)

CInqycomm::CInqycomm()
{
	// initialize the CICS COMMAREA
	memset(&inqy, '\0', sizeof(inqy));
	memset(&brws, '\0', sizeof(brws));
}

CInqycomm::~CInqycomm()
{
}


//BEGIN_MESSAGE_MAP(CInqycomm, CCmdTarget)
	//{{AFX_MSG_MAP(CInqycomm)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
//END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInqycomm message handlers

//////////////////////////////////////////////////////////////////////
//  Sets the Commarea for inquires and executes the CICS transaction
//////////////////////////////////////////////////////////////////////

int CInqycomm::executeInquiry(Cvartable& vars, int *usrcalen, void *usrca)

{
	CString cics_client_name = vars.GetVarPtr("CICSCLIENT");
	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString cics_message;
	CString cics_abcode;
	int rc=0;
	int	calen = INQUIRECOMMSIZE;
	const char* chgbutton = CHGBUTTON;

	// Trace point for debugging
	ISAPITRACE("Entering executeInquiry\n");

	// Map data length
	int	maplen = 0;

	// Clear the data area to nulls
	memset(&inqy, '\0', sizeof(INQYCOMM));

	// Set the BMS map fields
	maplen = setMenuMapData(vars);

	// set the transaction code 
	// note that we will use the update code, so that we get back
	// a user comm area with the current record status. This allows
	// the user to change the data and then press the update button.
	// set the user COMMAREA length to zero
	CEcisubs::setStandardHeader(&inqy.comm_head, calen, maplen, MENUMAPNAME, INQUIRETRAN, cics_client_name);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	rc = CEcisubs::executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy, 
	                            cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	// Get the data from the CICS COMMAREA and update the instance variables
	getDataFromComm(rc, cics_message, cics_abcode, vars);

	if (rc == 0)
	{
		// return the new user COMMAREA
		*usrcalen = CComsubs::comm_to_int(inqy.comm_head.USRCALEN, sizeof(inqy.comm_head.USRCALEN));
		if (*usrcalen > 0)
		{
			// move the new user COMMAREA to passed area
			memcpy(usrca, &inqy.user_comm, *usrcalen);

			// Set the Change button variable so that a change button will be displayed
			vars.AddVar("CHANGEBUTTON", chgbutton);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	return rc;
}

//////////////////////////////////////////////////////////////////////
// Routine to handle update requests
//////////////////////////////////////////////////////////////////////

int CInqycomm::executeChange(Cvartable& vars, int *usrcalen, void *usrca)

{
	CString cics_client_name = vars.GetVarPtr("CICSCLIENT");
	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString old_account = vars.GetVarPtr("OLDACCTNO");
	CString acctno = vars.GetVarPtr("ACCTNO");
	CString cics_message;
	CString cics_abcode;
	int rc=0;
	int	calen=0;

	// Trace point for debugging
	ISAPITRACE("Entering executeChange\n");

	// Variables for user COMMAREA length and BMS map data length
	int maplen = 0;

	// Clear the data area to nulls
	memset(&inqy, '\0', sizeof(INQYCOMM));

	// Get a pointer to the commarea
	// set the user input data                   
	// the user program expects binary zeros     
	// in any unmodified fields.  Since the      
	// commarea was initialized to binary zeros  
	// no action is necessary on unmodified      
	// fields.  Fields will be compared to their 
	// original values to see if they have been  
	// changed.                                  
	// To update a record requires two user      
	// interactions.  In the first one, the user 
	// enters the account number and selects     
	// update.  The account information is       
	// displayed, and the oldaccount field is    
	// set as a hidden variable in the HTML      
	// page.  The user then changes the data and 
	// selects update again.  This time, the     
	// fields which have been changed are        
	// updated in the file.  The first decision  
	// is to determine whether this is the first 
	// interaction for this account number or    
	// second.                                   

	// Check if we are updating an account that has already been displayed
	// If the accounts match, then proceed with the update, otherwise
	// display the data for the new account number, as an inquiry
	if (acctno.Compare(old_account) == 0)
	{
		// second interaction, try to update the  
		// account data.                          
		// Set the map data and clear the COMMAREA
		maplen = setDetailMapData(vars);

		// Set the User COMMAREA if there is one
		if (*usrcalen > 0)
		{
			// Get the expected COMMAREA length
			calen = sizeof(inqy.user_comm);
			// Is the actual user COMMAREA smaller?
			if (*usrcalen < calen)
			{
				// Set the length to the saved COMMAREA
				calen = *usrcalen;
			}

			// move the user COMMAREA to the real COMMAREA
			memcpy(&inqy.user_comm, usrca, calen);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	if (rc == 0)
	{
		// set the user commarea length 
		// for updates, the host program expects 80 bytes 
		CEcisubs::setStandardHeader(&inqy.comm_head, calen, maplen, DETAILMAPNAME, CHANGETRAN, cics_client_name);
	}
	else
	{
		// first interaction, display the account 
		// information for the user to change.    
		// move the account number to the keyi field 
		maplen = setMenuMapData(vars);

		// set the user commarea length to indicate 
		//the first pass.                          
		// The COMMAREA length is set to 0 for an inquiry,
		// since this is what the host program expects
		// Build the standard header part of the COMMAREA
		CEcisubs::setStandardHeader(&inqy.comm_head, calen, maplen, MENUMAPNAME, INQUIRETRAN, cics_client_name);
	}

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	rc = CEcisubs::executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy,
								cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	// Get the data from the CICS COMMAREA and update the instance variables
	getDataFromComm(rc, cics_message, cics_abcode, vars);

	if (rc == 0)
	{
		// return the new user COMMAREA
		*usrcalen = CComsubs::comm_to_int(inqy.comm_head.USRCALEN, sizeof(inqy.comm_head.USRCALEN));
		if (*usrcalen > 0)
		{
			// move the new user COMMAREA to passed area
			memcpy(usrca, &inqy.user_comm, *usrcalen);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	return rc;
}

//////////////////////////////////////////////////////////////////////
// Routine to handle add requests
// Add requests will be done in two passes, as is required by 
// 3270 version of the program.  The first pass will send in the
// add request and account number using a MENU map, and the second
// will send in the rest of the data using a DETAIL map.
// The CICS application will load the user COMMAREA after the first
// request.
//////////////////////////////////////////////////////////////////////

int CInqycomm::executeAdd(Cvartable& vars, int *usrcalen, void *usrca)

{
	CString cics_client_name = vars.GetVarPtr("CICSCLIENT");
	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString cics_message;
	CString cics_abcode;
	CString nxtTran;
	int rc=0;
	int	calen=0;

	// work variable
	CString vardata;

	// Trace point for debugging
	ISAPITRACE("Entering executeAdd\n");

	// BMS data length
	int maplen = 0;

	// Set the User COMMAREA if there is one
	if (*usrcalen > 0)
	{
		// Get the expected COMMAREA length
		calen = sizeof(inqy.user_comm);
		// Is the actual user COMMAREA smaller?
		if (*usrcalen < calen)
		{
			// Set the length to the saved COMMAREA
			calen = *usrcalen;
		}

		// move the user COMMAREA to the real COMMAREA
		memcpy(&inqy.user_comm, usrca, calen);
	}

	// Clear the data area to nulls
	memset(&inqy, '\0', sizeof(INQYCOMM));

	// Set the BMS map fields for the first pass
	maplen = setMenuMapData(vars);

	// Set the standard COMMAREA header for the first pass of an add request
	// There is no user COMMAREA at this point, so we pass a zero for the length
	CEcisubs::setStandardHeader(&inqy.comm_head, 0, maplen, MENUMAPNAME, ADDTRAN, cics_client_name);

	// Execute the CICS MENU Transaction to prepare for the real add transaction
	rc = CEcisubs::executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy,
		                            cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	// Get the next transid to check if it is what we expect
	CComsubs::from_comm(nxtTran, inqy.comm_head.NXTRNID, sizeof(inqy.comm_head.NXTRNID));

	// Check if the ECI call worked and we received the right data back
	if ((rc == Ccl::noError) && (nxtTran.Compare(ADDTRAN) == 0))
	{
		// Trace point for debugging
		ISAPITRACE("Entering second phase of add\n");

		// Second pass for adding a record
		// Set the input fields in the COMMAREA
		maplen = setDetailMapData(vars);

		// set the transaction code and other standard header fields 
		// We can leave most of the COMMAREA alone, including the user COMMAREA
		//CEcisubs::setStandardHeader(&inqy.comm_head, calen, maplen, DETAILMAPNAME, ADDTRAN, cics_client_name);
		// Set the User Transaction Name field in the COMMAREA from the Next Transid field
		CComsubs::to_comm(nxtTran, inqy.comm_head.USRTRNID, sizeof(inqy.comm_head.USRTRNID), ALPHA_FIELD);

		// set the BMS total data length 
		CComsubs::int_to_comm(maplen, inqy.comm_head.USRMAPLN,sizeof(inqy.comm_head.USRMAPLN));

		// Initialize the return code and next transaction id
		memset(inqy.comm_head.RETCODE, '0', sizeof(inqy.comm_head.RETCODE));
		memset(inqy.comm_head.NXTRNID, ' ', sizeof(inqy.comm_head.NXTRNID));

		// End of COMMAREA build

		// Execute the CICS Transaction
		int rc = CEcisubs::executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy,
			                            cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	}

	// Get the data from the CICS COMMAREA and update the instance variables
	getDataFromComm(rc, cics_message, cics_abcode, vars);

	if (rc == 0)
	{
		// return the new user COMMAREA
		*usrcalen = CComsubs::comm_to_int(inqy.comm_head.USRCALEN, sizeof(inqy.comm_head.USRCALEN));
		if (*usrcalen > 0)
		{
			// move the new user COMMAREA to passed area
			memcpy(usrca, &inqy.user_comm, *usrcalen);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	return rc;
}

int CInqycomm::setDetailMapData(Cvartable& vars)

{
	// work variable
	CString vardata;

	// Trace point for debugging
	ISAPITRACE("Entering setDetailMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// Set the BMS fields
	vardata = vars.GetVarPtr("ACCTNO");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.numbi,
						 &inqy.detail.dfhdgbi.numbl,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.numbi),
						 NUMERIC_FIELD);

	vardata = vars.GetVarPtr("NAME");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.namei,
						 &maplen,
						 &inqy.detail.dfhdgbi.namel,
						 sizeof(inqy.detail.dfhdgbi.namei),
						 ALPHA_FIELD);

	vardata = vars.GetVarPtr("ADDRESS");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.addri,
						 &inqy.detail.dfhdgbi.addrl,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.addri),
						 ALPHA_FIELD);

	vardata = vars.GetVarPtr("PHONE");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.phonei,
						 &inqy.detail.dfhdgbi.phonel,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.phonei),
						 ALPHA_FIELD);

	vardata = vars.GetVarPtr("DATE");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.datei,
						 &inqy.detail.dfhdgbi.datel,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.datei),
						 ALPHA_FIELD);

	vardata = vars.GetVarPtr("AMOUNT");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.amounti,
						 &inqy.detail.dfhdgbi.amountl,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.amounti),
						 ALPHA_FIELD);

	vardata = vars.GetVarPtr("COMMENT");
	CComsubs::toBMSfield(vardata,
						 inqy.detail.dfhdgbi.commenti,
						 &inqy.detail.dfhdgbi.commentl,
						 &maplen,
						 sizeof(inqy.detail.dfhdgbi.commenti),
						 ALPHA_FIELD);

	return maplen;
}

int CInqycomm::setMenuMapData(Cvartable& vars)

{
	// work variable
	CString vardata;

	// Trace point for debugging
	
	ISAPITRACE("Entering setMenuMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// set the account number from the instance variable
	vardata = vars.GetVarPtr("ACCTNO");
	CComsubs::toBMSfield(vardata,
						 inqy.menu.dfhdgai.keyi,
						 &inqy.menu.dfhdgai.keyl,
						 &maplen,
						 sizeof(inqy.menu.dfhdgai.keyi),
						 NUMERIC_FIELD);

	return maplen;
}

int CInqycomm::setBrwsMenuData(Cvartable& vars)

{
	// work variable
	CString vardata;

	// Trace point for debugging
	ISAPITRACE("Entering setBrwsMenuData\n");

	// Total length of all input data in map
	short maplen = 0;

	// set the account number from the instance variable
	vardata = vars.GetVarPtr("ACCTNO");
	CComsubs::toBMSfield(vardata,
						 brws.menu.dfhdgao.keyo,
						 &brws.menu.dfhdgai.keyl,
						 &maplen,
						 sizeof(brws.menu.dfhdgao.keyo),
						 NUMERIC_FIELD);

	return maplen;
}

int CInqycomm::setBrwsMapData(char brtype)

{
	// work variable
	CString vardata;

	// Trace point for debugging
	ISAPITRACE("Entering setBrwsMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// set the account number from the instance variable
	vardata = brtype;
	CComsubs::toBMSfield(vardata,
						 &brws.browse.dfhdgco.diro,
						 &brws.browse.dfhdgci.dirl,
						 &maplen,
						 sizeof(brws.browse.dfhdgco.diro),
						 NUMERIC_FIELD);

	return maplen;
}

void CInqycomm::getDataFromComm(int RC, CString& cics_message, CString& cics_abcode, Cvartable& vars)

{
	// work variables
	CString vardata;
	char RetCode[20];

	// Trace point for debugging
	ISAPITRACE("Entering getDataFromComm\n");

	if (RC == Ccl::noError)
	{
		// Check for a DETAIL map
		if (memcmp(inqy.comm_head.MAPNAME,"DETAIL  ",
			       sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			// Populate the dialog field variables
			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.namei,
									sizeof(inqy.detail.dfhdgbi.namei));
			vars.UpdateVar("NAME", vardata);

			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.addri,
								sizeof(inqy.detail.dfhdgbi.addri));
			vars.UpdateVar("ADDRESS", vardata);

			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.phonei,
								sizeof(inqy.detail.dfhdgbi.phonei));
			vars.UpdateVar("PHONE", vardata);

			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.datei,
								sizeof(inqy.detail.dfhdgbi.datei));
			vars.UpdateVar("DATE", vardata);

			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.amounti,
								sizeof(inqy.detail.dfhdgbi.amounti));
			vars.UpdateVar("AMOUNT", vardata);

			CComsubs::from_comm(vardata, inqy.detail.dfhdgbi.commenti,
								sizeof(inqy.detail.dfhdgbi.commenti));
			vars.UpdateVar("COMMENT", vardata);

			// Set the old account number, to allow changes
			vardata = vars.GetVarPtr("ACCTNO");
			vars.UpdateVar("OLDACCOUNT", vardata);


			if ((inqy.detail.dfhdgbo.msg1o[0] != '\0') &&
				(inqy.detail.dfhdgbo.msg1o[0] != ' '))
			{
				CComsubs::from_comm(vardata, inqy.detail.dfhdgbo.msg1o,
									sizeof(inqy.detail.dfhdgbo.msg1o));
				vars.UpdateVar("ERRMSG", vardata);
			}

			if ((inqy.detail.dfhdgbo.msg3o[0] != '\0') &&
				(inqy.detail.dfhdgbo.msg3o[0] != ' '))
			{
				CComsubs::from_comm(vardata, inqy.detail.dfhdgbo.msg3o,
									sizeof(inqy.detail.dfhdgbo.msg3o));
				vars.UpdateVar("ERRMSG", vardata);
			}
		}

		// Check for a MENU map
		if (memcmp(inqy.comm_head.MAPNAME,"MENU    ",
				   sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			CComsubs::from_comm(vardata, inqy.menu.dfhdgao.msgo,
								sizeof(inqy.menu.dfhdgao.msgo));
			vars.UpdateVar("ERRMSG", vardata);
		}

		// Check for a CICS SEND TEXT
		if (memcmp(inqy.comm_head.MAPNAME,"SENDTEXT",
				   sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			CComsubs::from_comm(vardata, inqy.comm_head.textarea,
								sizeof(inqy.comm_head.textarea));
			vars.UpdateVar("ERRMSG", vardata);
		} // endif
	}
	else
	{
		// Check for invalid user id and password 
		// If the mirror program detects an  invalid user id and/or 
		// password, it will issue an AEY7 abend 
		if ((RC == Ccl::transaction) && (strcmp(cics_abcode, "AEY7") == 0))
		{
		    // Set a variable with the error message 
			vardata = "Invalid Userid or Password";
		}
		else
		{
			// set the ECI error message and abend code
			itoa(RC, RetCode, 10);
			vardata = RetCode;
			vars.UpdateVar("ECIERRCODE", vardata);
			vars.UpdateVar("ECIERRMSG", cics_message);
			vars.UpdateVar("ECIERRABCODE", cics_abcode);

			// set the user message field to indicate a CICS error
			vardata = "Error calling CICS";
		}

		// Set the error message
		vars.UpdateVar("ERRMSG", vardata);

		// Set two formatting variables
		vardata = "<HR>";
		vars.UpdateVar("ECIERRHR", vardata);
		vardata = "<P>";
		vars.UpdateVar("ECIERRNL", vardata);
	}
}

int CInqycomm::executeStartBrowse(Cvartable& vars, int *usrcalen, void *usrca, CString& mapname)

{
	CString cics_client_name = vars.GetVarPtr("CICSCLIENT");
	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString cics_message;
	CString cics_abcode;
	int rc=0;
	int	calen=STARTBRCOMMSIZE;

	// Trace point for debugging
	ISAPITRACE("Entering StartBrowse\n");

	// Map data length
	int	maplen = 0;

	// Clear the data area to nulls
	memset(&brws, '\0', sizeof(BRWSCOMM));

	// Set the BMS map fields and clear the COMMAREA
	maplen = setBrwsMenuData(vars);

	// set the standard fields in the commarea header 
	// Note that the user COMMAREA size must be 0 for a start browse
	// This is what the CICS program expects
	// Therefore, we ignore any current user commarea
	CEcisubs::setStandardHeader(&brws.comm_head, calen, maplen, MENUMAPNAME, BROWSETRAN, cics_client_name);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the cics variables
	rc = CEcisubs::executeCICSTran(BRWSPGM, sizeof(brws), &brws, 
								cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	// Check if it worked
	if (rc == Ccl::noError)
	{
		// Get the user data from the COMMAREA
		getBrwsData(vars);

		// Return the map name
		CComsubs::from_comm(mapname, brws.comm_head.MAPNAME,
							sizeof(brws.comm_head.MAPNAME));	
	}
	else
	{
		// get the ECI error data into variables
		getDataFromComm(rc, cics_message, cics_abcode, vars);
	}

	if (rc == 0)
	{
		// return the new user COMMAREA
		*usrcalen = CComsubs::comm_to_int(brws.comm_head.USRCALEN, sizeof(brws.comm_head.USRCALEN));
		if (*usrcalen > 0)
		{
			// move the new user COMMAREA to passed area
			memcpy(usrca, &brws.brws_comm, *usrcalen);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	return rc;
}

int CInqycomm::executeBrowseNext(char brtype, Cvartable& vars, int *usrcalen, void *usrca, CString& mapname)

{
	CString cics_client_name = vars.GetVarPtr("CICSCLIENT");
	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString cics_message;
	CString cics_abcode;
	int rc=0;
	int	calen=0;

	// Trace point for debugging
	ISAPITRACE("Entering executeBrowseNext\n");

	// Map data length
	int	maplen = 0;

	// Clear the data area to nulls
	memset(&brws, '\0', sizeof(BRWSCOMM));

	// Set the BMS map direction field and length and clear the COMMAREA
	maplen = setBrwsMapData(brtype);

	// Set the User COMMAREA if there is one
	if (*usrcalen > 0)
	{
		// Get the expected COMMAREA length
		calen = sizeof(brws.brws_comm);

		// Is the actual user COMMAREA smaller?
		if (*usrcalen < calen)
		{
			// Set the length to the saved COMMAREA
			calen = *usrcalen;
		}

		// move the user COMMAREA to the real COMMAREA
		memcpy(&brws.brws_comm, usrca, calen);
	}

	// set the standard fields in the commarea header 
	CEcisubs::setStandardHeader(&brws.comm_head, calen, maplen, BROWSEMAPNAME, BROWSETRAN, cics_client_name);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the cics variables
	rc = CEcisubs::executeCICSTran(BRWSPGM, sizeof(brws), &brws, 
								cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	// Check if it worked
	if (rc == Ccl::noError)
	{
		// Get the user data from the COMMAREA
		getBrwsData(vars);

		// Return the map name
		CComsubs::from_comm(mapname, brws.comm_head.MAPNAME,
							sizeof(brws.comm_head.MAPNAME));	
	}
	else
	{
		// get the ECI error data into variables
		getDataFromComm(rc, cics_message, cics_abcode, vars);
	}

	if (rc == 0)
	{
		// return the new user COMMAREA
		*usrcalen = CComsubs::comm_to_int(brws.comm_head.USRCALEN, sizeof(brws.comm_head.USRCALEN));
		if (*usrcalen > 0)
		{
			// move the new user COMMAREA to passed area
			memcpy(usrca, &brws.brws_comm, *usrcalen);
		}
	}
	else
	{
		// Error calling CICS - set the COMMAREA length to zero
		*usrcalen = 0;
	}

	return rc;
}

void CInqycomm::getBrwsData(Cvartable & vars)

{
	// work variables
	CString vardata;

	// Trace point for debugging
	ISAPITRACE("Entering getBrwsData\n");

	// Check for a MENU map
	if (memcmp(brws.comm_head.MAPNAME,"MENU    ",
			   sizeof(brws.comm_head.MAPNAME)) == 0)
	{
		CComsubs::from_comm(vardata, brws.menu.dfhdgao.msgo,
							sizeof(brws.menu.dfhdgao.msgo));
		vars.UpdateVar("ERRMSG", vardata);
	}

	// Check for a CICS SEND TEXT
	if (memcmp(brws.comm_head.MAPNAME,"SENDTEXT",
			   sizeof(brws.comm_head.MAPNAME)) == 0)
	{
		CComsubs::from_comm(vardata, brws.comm_head.textarea,
							sizeof(brws.comm_head.textarea));
		vars.UpdateVar("ERRMSG", vardata);
	} // endif

   if (memcmp(brws.comm_head.MAPNAME,"BROWSE  ",
              sizeof(brws.comm_head.MAPNAME)) == 0)
   {
		CComsubs::from_comm(vardata, brws.browse.dfhdgco.number1o,
							sizeof(brws.browse.dfhdgco.number1o));
		vars.UpdateVar("NUMBER1", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.name1o,
							sizeof(brws.browse.dfhdgco.name1o));
		vars.UpdateVar("NAME1", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.amount1o,
							sizeof(brws.browse.dfhdgco.amount1o));
		vars.UpdateVar("AMOUNT1", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.number2o,
							sizeof(brws.browse.dfhdgco.number2o));
		vars.UpdateVar("NUMBER2", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.name2o,
							sizeof(brws.browse.dfhdgco.name2o));
		vars.UpdateVar("NAME2", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.amount2o,
							sizeof(brws.browse.dfhdgco.amount2o));
		vars.UpdateVar("AMOUNT2", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.number3o,
							sizeof(brws.browse.dfhdgco.number3o));
		vars.UpdateVar("NUMBER3", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.name3o,
							sizeof(brws.browse.dfhdgco.name3o));
		vars.UpdateVar("NAME3", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.amount3o,
							sizeof(brws.browse.dfhdgco.amount3o));
		vars.UpdateVar("AMOUNT3", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.number4o,
							sizeof(brws.browse.dfhdgco.number4o));
		vars.UpdateVar("NUMBER4", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.name4o,
							sizeof(brws.browse.dfhdgco.name4o));
		vars.UpdateVar("NAME4", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.amount4o,
							sizeof(brws.browse.dfhdgco.amount4o));
		vars.UpdateVar("AMOUNT4", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.msg2o,
							sizeof(brws.browse.dfhdgco.msg2o));
		vars.UpdateVar("ERRMSG", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.msg0o,
							sizeof(brws.browse.dfhdgco.msg0o));
		vars.UpdateVar("ERRMSG2", vardata);

		CComsubs::from_comm(vardata, brws.browse.dfhdgco.msg1o,
							sizeof(brws.browse.dfhdgco.msg1o));
		vars.UpdateVar("ERRMSG3", vardata);
   }
}

/////////////////////////////////////////////////
//  End of file
/////////////////////////////////////////////////
