/**************************************************************/
/*                                                            */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required includes for C functions                          */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#define CICS_W32
#include <cics_eci.h>

/**************************************************************/
/* Required include for Common subroutines                    */
/**************************************************************/

#include "comsubs.h"

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#include "ecimsg.h"

/**************************************************************/
/* Routine to print CICS ECI error messages                   */
/**************************************************************/

char* Print_ECI_Error(int rc)
{
	char *msgptr;

        switch (rc)
         {
          case ECI_ERR_INVALID_DATA_LENGTH:
            msgptr = &"Invalid Data Length";
            break;

          case ECI_ERR_INVALID_EXTEND_MODE:
            msgptr = &"Invalid Extend Mode";
            break;

          case ECI_ERR_NO_CICS:
            msgptr = &"CICS is not running";
            break;

          case ECI_ERR_CICS_DIED:
            msgptr = &"CICS terminated unexpectedly";
            break;

          case ECI_ERR_REQUEST_TIMEOUT:
            msgptr = &"Request timed out";
            break;

          case ECI_ERR_RESPONSE_TIMEOUT:
            msgptr = &"Response timed out";
            break;

          case ECI_ERR_TRANSACTION_ABEND:
            msgptr = &"Transaction abended";
            break;

          case ECI_ERR_EXEC_NOT_RESIDENT:
            msgptr = &"Not resident";
            break;

          case ECI_ERR_SYSTEM_ERROR:
            msgptr = &"System error";
            break;

          case ECI_ERR_NULL_WIN_HANDLE:
            msgptr = &"Window handle cannot be null";
            break;

          case ECI_ERR_NULL_MESSAGE_ID:
            msgptr = &"Invalid message id";
            break;

          case ECI_ERR_THREAD_CREATE_ERROR:
            msgptr = &"Unable to create thread";
            break;

          case ECI_ERR_INVALID_CALL_TYPE:
            msgptr = &"Unknown call type";
            break;

          case ECI_ERR_ALREADY_ACTIVE:
            msgptr = &"Another request is already active";
            break;

          case ECI_ERR_RESOURCE_SHORTAGE:
            msgptr = &"A resource shortage occured";
            break;

          case ECI_ERR_NO_SESSIONS:
            msgptr = &"No sessions are available";
            break;

          case ECI_ERR_NULL_SEM_HANDLE:
            msgptr = &"The semaphore handle cannot be null";
            break;

          case ECI_ERR_INVALID_DATA_AREA:
            msgptr = &"Invalid data area pointer";
            break;

          case ECI_ERR_INVALID_VERSION:
            msgptr = &"Invalid version number";
            break;

          case ECI_ERR_UNKNOWN_SERVER:
            msgptr = &"Cannot find the named CICS server";
            break;

          case ECI_ERR_CALL_FROM_CALLBACK:
            msgptr = &"Callback routine cannot use ECI calls";
            break;

          case ECI_ERR_INVALID_TRANSID:
            msgptr = &"Transaction not found on CICS server";
            break;

          case ECI_ERR_MORE_SYSTEMS:
            msgptr = &"More systems error";
            break;

          case ECI_ERR_NO_SYSTEMS:
            msgptr = &"No systems are available";
            break;

          case ECI_ERR_SECURITY_ERROR:
            msgptr = &"Security error";
            break;

          case ECI_ERR_MAX_SYSTEMS:
            msgptr = &"Max systems exceeded";
            break;

          case ECI_ERR_MAX_SESSIONS:
            msgptr = &"Max sessions exceeded";
            break;

          case ECI_ERR_ROLLEDBACK:
            msgptr = &"CICS transaction was rolled back unexpectedly";
            break;

          default:
            msgptr = &"Unknown error";
            break;
         }

	return msgptr;
}

/**************************************************************/
/* Routine to execute a CICS ECI call                         */
/**************************************************************/

int performECI(char *pgmName, char *userid, char *pw, char *sysid, char *pCommArea,
			   int commLen, VARSTRUCT *pVars, char *abendcode)
{
    ECI_PARMS   eciParms;
	int			rc;
	char		*msgptr;
	char		vardata[MAX_DATA_LEN];

/**************************************************************/
/* Initialize variables                                       */
/**************************************************************/

	memset(abendcode, '\0', ABCODE_LEN + 1);

/**************************************************************/
/* Build the ECI block                                        */
/**************************************************************/

    memset(&eciParms,'\0',sizeof(eciParms));
    setlocale(LC_ALL,"");
    eciParms.eci_call_type = ECI_SYNC;
    eciParms.eci_version = ECI_VERSION_1;
    eciParms.eci_luw_token = 0;
    eciParms.eci_extend_mode = ECI_NO_EXTEND;

	/* Set the user id and password */
    memcpy(eciParms.eci_userid,userid,MAX_PW_LEN);
    memcpy(eciParms.eci_password,pw,MAX_PW_LEN);

	/* Set up the CICS commarea */
    eciParms.eci_commarea = pCommArea;
    eciParms.eci_commarea_length = commLen;

	if ((sysid != NULL) && (sysid[0] != '\0'))
	{
		memcpy(eciParms.eci_system_name,sysid,8);
	}

	/* Set the program name */
    memcpy(eciParms.eci_program_name,pgmName,8);

	/* Execute the request */
    rc = CICS_ExternalCall(&eciParms);

	/* Check for specific types of error codes with additional useful info */
     if (rc != ECI_NO_ERROR)
     {
		 /* Create a variable with the ECI error message */
		msgptr = Print_ECI_Error(rc);
		AddVar(ECI_ERR_MSG, msgptr, pVars);

		/* Get the ECI error code in a message */
        sprintf(vardata, " ECI Return Code %d", rc);
		AddVar(ECI_ERR_CODE, vardata, pVars);

		/* Create Ending and Beginning Text */
		AddVar(ECI_ERR_HR, ECI_HR, pVars);
		AddVar(ECI_ERR_NL, ECI_NEWLINE, pVars);

        if (rc == ECI_ERR_SYSTEM_ERROR)
         {
			/* Get the system error if it exists */
            sprintf(vardata, " System Error Code %d", eciParms.eci_sys_return_code);
			AddVar(ECI_ERR_SYSERR, vardata, pVars);
         }

        if ((rc == ECI_ERR_TRANSACTION_ABEND) && (eciParms.eci_abend_code[0] != 0))
         {
			/* Get the abend code if it exists */
            //memcpy(abcode, eciParms.eci_abend_code, ABCODE_LEN);
            memcpy(abendcode, eciParms.eci_abend_code, ABCODE_LEN);
            sprintf(vardata, " CICS Abend Code %s", abendcode);
			AddVar(ECI_ERR_ABCODE, vardata, pVars);
         }
     }

	return rc;
}

/****************/
/* End of File  */
/****************/