// LOGISA.CPP - Implementation file for your Internet Server
//    logisa Extension

#include "stdafx.h"
#include "logisa.h"

// Name of the HTML prototype file 
#define HTML_LOGON_FILE	"\\inetpub\\scripts\\isalogon.htm"
#define HTML_INQY_FILE	"\\inetpub\\scripts\\isainqy.htm"

// Default temporary state file directory location
// Note that this can be overriden by using the CICSTEMP
// environment variable.
#define CICSTEMP "c:\\CICSTEMP"
#define MAX_CICSTEMP_LEN MAX_DATA_LEN - 31
#define FILE_PREFIX "CICS"

#define TRACE_FILE_NAME "C:\\TEMP\\CICSISA.TRC"

/* CICS program name */
#define CICSPROG "VERIFYPW"

// 3270 field types 
#define ALPHA_FIELD   0
#define NUMERIC_FIELD 1
#define CICS_FIELD    2

// CICS Status file header structure
typedef struct {
   char userName[8];
   char passWord[8];
   char sysName[8];
   char clientName[4];
   char replayCount[8];
   char tctuaLength[4];
   char caLength[8];
} STATUS_FILE_HEADER;

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;

///////////////////////////////////////////////////////////////////////
// command-parsing map

BEGIN_PARSE_MAP(CLogisaExtension, CHttpServer)
	// TODO: insert your ON_PARSE_COMMAND() and 
	// ON_PARSE_COMMAND_PARAMS() here to hook up your commands.
	// For example:

	ON_PARSE_COMMAND(procTest, CLogisaExtension, ITS_PSTR)
	ON_PARSE_COMMAND_PARAMS("SUBMIT")
	ON_PARSE_COMMAND(procLogon, CLogisaExtension, ITS_PSTR ITS_PSTR ITS_PSTR ITS_PSTR ITS_PSTR) 
	ON_PARSE_COMMAND_PARAMS("CICSUSER CICSPW CICSSYS CICSCLIENT SUBMIT")
	ON_PARSE_COMMAND(Default, CLogisaExtension, ITS_EMPTY)
	DEFAULT_PARSE_COMMAND(Default, CLogisaExtension)
END_PARSE_MAP(CLogisaExtension)


///////////////////////////////////////////////////////////////////////
// The one and only CLogisaExtension object

CLogisaExtension theExtension;

fixCclECI *pECI;

///////////////////////////////////////////////////////////////////////
// CLogisaExtension implementation

CLogisaExtension::CLogisaExtension()
{
	pECI = new fixCclECI();
}

CLogisaExtension::~CLogisaExtension()
{
}

BOOL CLogisaExtension::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
	// Call default implementation for initialization
	CHttpServer::GetExtensionVersion(pVer);

	// Load description string
	TCHAR sz[HSE_MAX_EXT_DLL_NAME_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_SERVER, sz, HSE_MAX_EXT_DLL_NAME_LEN));
	_tcscpy(pVer->lpszExtensionDesc, sz);
	return TRUE;
}

///////////////////////////////////////////////////////////////////////
// CLogisaExtension command handlers

void CLogisaExtension::Default(CHttpServerContext* pCtxt)
{
	Cvartable	vars;
    char		*filename;
	int			rc;
	char   writerr[100];

	// Set the file name to the LOGIN HTML page
    filename = HTML_LOGON_FILE;

	// write the default LOGON HTML page
	rc = vars.write_html_page(pCtxt, filename);

	// Check if it worked
	if (rc != 0)
	{
		// failed - we need to send an error message so the
		// user knows what happened
		// write the HTTP headers
		StartContent(pCtxt);
		WriteTitle(pCtxt);
		// Write an Error Message
		*pCtxt << "Error writing page\n";
		sprintf(writerr, "Return code is %d\n", rc);
		*pCtxt << writerr; 
		// Finish the HTML page
		EndContent(pCtxt);
	}
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CLogisaExtension, CHttpServer)
	//{{AFX_MSG_MAP(CLogisaExtension)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0



///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/

void CLogisaExtension::build_commarea(Cvartable & vars, VERIFYCOMM * pCommArea)

{
	// work variable
	CString vardata;

	// Set the version number 
	memcpy(pCommArea->VERSION,"00",sizeof(pCommArea->VERSION));

	// Fill in the userid and password 
	vardata = vars.GetVarPtr("CICSUSER");
    to_comm(vardata, pCommArea->CICSUSER,
            sizeof(pCommArea->CICSUSER),
            ALPHA_FIELD);

	vardata = vars.GetVarPtr("CICSPW");
    to_comm(vardata, pCommArea->CICSPW,
            sizeof(pCommArea->CICSPW),
            ALPHA_FIELD);

	vardata = vars.GetVarPtr("CICSCLIENT");
    to_comm(vardata, pCommArea->CLIENTNAME,
            sizeof(pCommArea->CLIENTNAME),
            ALPHA_FIELD);
}

int CLogisaExtension::performLogon(Cvartable & vars, int * RCode)

{
	// work variables
	CString vardata;
	char RetCode[20];

	// initialize RCode return variable
	*RCode = 0;

	// CICS COMMAREA for password verification program
	VERIFYCOMM pCommArea;

	CString cics_sys = vars.GetVarPtr("CICSSYS");
	CString cics_user = vars.GetVarPtr("CICSUSER");
	CString cics_pw = vars.GetVarPtr("CICSPW");
	CString cics_message;
	CString cics_abcode;

	// initialize the CICS COMMAREA to binary zeros
	memset((void *) &pCommArea, '\0', sizeof(pCommArea));

	// set the COMMAREA fields
	build_commarea(vars, &pCommArea);

	// Execute the CICS Transaction and update the instance variables
	int rc = executeCICSTran(CICSPROG, sizeof(pCommArea), &pCommArea, 
		                            cics_sys, cics_user, cics_pw, cics_message, cics_abcode);

	if (rc == Ccl::noError)
	{
		// Check the CICS resp code from the EXEC CICS VERIFY PASSWORD
		memset(RetCode, '\0', sizeof(RetCode));
		memcpy(pCommArea.RESP, RetCode, sizeof(pCommArea.RESP));
		*RCode = atoi(RetCode);
	}
	else
	{
		// Check for invalid user id and password 
		// If the mirror program detects an  invalid user id and/or 
		// password, it will issue an AEY7 abend 
		if ((rc == Ccl::transaction) && (strcmp(cics_abcode, "AEY7") == 0))
		{
		    // Set a variable with the error message 
			vardata = "Invalid Userid or Password";
		}
		else
		{
			// set the ECI error message and abend code
			itoa(rc, RetCode, 10);
			vardata = RetCode;
			vars.UpdateVar("ECIERRCODE", vardata);
			vars.UpdateVar("ECIERRMSG", cics_message);
			vars.UpdateVar("ECIERRABCODE", cics_abcode);

			// set the user message field to indicate a CICS error
			vardata = "Error calling CICS";
		}

		// Add the error message to the variable table
		vars.UpdateVar("ERRMSG", vardata);

		// Set two formatting variables
		vardata = "<HR>";
		vars.UpdateVar("ECIERRHR", vardata);
		vardata = "<P>";
		vars.UpdateVar("ECIERRNL", vardata);
	}

	return rc;
}

////////////////////////////////////////////////////////////////
// Routine to get the value of a variable (CString) and then  
//  copy it into the CICS commarea.                           
////////////////////////////////////////////////////////////////

void CLogisaExtension::to_comm(CString & submit, char * CommField, int maxLen, int isNumber)

{
   int     len;
    LPTSTR  userdata;

    // Get the length of the CString data 
    len = submit.GetLength();

    // Get a pointer to the CString data 
    userdata = submit.GetBuffer(0);

    if (len > maxLen)
     {
       len = maxLen;
     }

    // set the field to blanks or zeros 
    if (isNumber == NUMERIC_FIELD)
    {
       // numeric field - initialize to zeros 
       memset(CommField,'0',maxLen);

       // copy the data, if there is any, right justified 
       if (len > 0)
       {
          memcpy(CommField + maxLen - len, userdata, len);
       }
    }
    else
    {
       // alpha field - initialize to blanks 
       if (isNumber == ALPHA_FIELD)
       {
          memset(CommField,' ',maxLen);
       }
       else
       {
          memset(CommField,'\0',maxLen);
       }

       // copy the data, if there is any, left justified 
       if (len > 0)
       {
          memcpy(CommField, userdata, len);
       }
    }

    // Release the buffer pointer 
    submit.ReleaseBuffer();
}

////////////////////////////////////////////////////////////////
// Routine to save an integer as a number string              
////////////////////////////////////////////////////////////////

void CLogisaExtension::int_to_comm(const long i, char * result, const int len)

{
   char temp[18];
   int maxlen;

   // Check input parameters
   ASSERT(result != NULL);
   ASSERT(len > 0);

   itoa(i, temp, 10);

   if (len < 12)
   {
      maxlen = len;
   }
   else
   {
      maxlen = 11;
   } // endif 

   if (maxlen > 0)
   {
      memset(result, '0', maxlen);
      if (strlen(temp) > 0)
      {
         memcpy(result + maxlen - strlen(temp), temp, strlen(temp));
      } // endif 
   } // endif 
}

////////////////////////////////////////////////////////////////
// Routine to execute a CICS transaction
////////////////////////////////////////////////////////////////

int CLogisaExtension::executeCICSTran(char * tran, 
									  int casize, 
									  void * dataArea, 
									  CString & cics_system, 
									  CString & cics_user, 
									  CString & cics_pw, 
									  CString & cics_message, 
									  CString & cics_abcode)
{
	// Trace point for debugging
	ISAPITRACE("Entering executeCICSTran\n");

	// Make sure we don't have a null pointer
	ISAPIASSERT(dataArea != NULL);

	// Make sure the COMMAREA size is > 0
	ISAPIASSERT(casize > 0);

	int RC = Ccl::noError;

	// Clear the previous CICS error message text
	cics_message.Empty();
	cics_abcode.Empty();

	try {	// Catch block for CclClass exceptions
		// Build a connection object
		CclConn connection(cics_system, cics_user, cics_pw);

		// Build a buffer object
		CclBuf buffer(casize, dataArea);

		// Build a flow object
		CclFlow sflow(Ccl::sync);

		// Execute the CICS transaction
		connection.link(sflow, tran, &buffer);
    }    // Catch CclClass exceptions
	catch ( CclException& exception ) 
	{
		// Trace point for debugging
		ISAPITRACE("CICS ECI error - %s\n",exception.exCodeText);

		RC = exception.exCode();
		cics_message = exception.exCodeText();

		// Check for CICS Transaction abend error
		// Is it a CICS transaction abend?
		if (RC == Ccl::transaction)
		{
			cics_abcode = pECI->m_Cclabcode;

			// Trace point for debugging
			ISAPITRACE("CICS transaction abend - %s\n", cics_abcode);
		}
	} // end catch

	return RC;
}

///////////////////////////////////////////////////////////////
//
// Routine to generate a unique temporary file name.
// This file will be used to save the CICS status on the
// web server.  The CICS status includes the user id and 
// password, system and client names, a counter used to
// detect replays and the latest CICS COMMAREA.
//
// The file name will be located in the C:\CICSTEMP 
// directory, unless the CICSTEMP environment variable
// is used to override this.  The file name will start
// with the characters CICS, and will include the CICS
// system and client names, followed by a temporary file
// name (number) to make the file unique.  This file name
// must be sent to the browser as a hidden variable, and 
// used to retrieve the file contents after the next 
// user input.
//
// The CICSTEMP environment variable must be less than
// 64 characters long.  This restriction is to prevent the
// total file name from exceeding the maximum variable table
// variable length of MAX_DATA_LEN.
//
///////////////////////////////////////////////////////////////

void CLogisaExtension::generateFilename(char* filename, 
										const char* clientname, 
										const char* systemname)

{
	// Work variables
	char	dirname[MAX_DATA_LEN + 1];
	char	*envvar;
	char	clname[5];
	char	sysname[9];
	char	fileprefix[31];
	char	*tempname;

	// initialize the work variables
	memset(dirname, 0, sizeof(dirname));
	memset(clname, 0, sizeof(clname));
	memset(sysname, 0, sizeof(sysname));

	// Initialize the file name
	filename[0] = 0;

	// Get the default directory name
	memcpy(dirname, CICSTEMP, sizeof(CICSTEMP));

	// check if the CICSTEMP environment variable is set
	envvar = getenv("CICSTEMP");
	if (envvar != NULL)
	{
		// We have one, check if it is too long
		if (strlen(envvar) <= MAX_CICSTEMP_LEN)
		{
			// replace the default drive and directory location
			strcpy(envvar, dirname);
		}
	}

	// Build the rest of the file name

	// Get the client and system names as strings, up to maximum lengths
	memcpy(clname, clientname, sizeof(clname) - 1);
	memcpy(sysname, systemname, sizeof(sysname) - 1);

	// Concate to the directory name
	// make sure we have a back slash character if we need one
	if ((strlen(dirname) > 0) && (dirname[strlen(dirname)] != '\\'))
	{
		strcat(dirname, "\\");
	}
	// build the file prefix
	fileprefix[0] = 0;
	strcat(fileprefix, FILE_PREFIX);	// Start name with CICS
	strcat(fileprefix, sysname);		// Add the system name
	strcat(fileprefix, clname);			// Add the client name

	// Now, generate the temporary file name
	tempname = _tempnam(dirname, fileprefix);

	// Check if it worked
	if (tempname != NULL)
	{
		// Copy the file name to the return area
		strcpy(filename, tempname);
		// Free the temporary file name area generated by _tempnam
		free(tempname);
	}
}

///////////////////////////////////////////////////////////////
//
// This routine will save the current CICS status in a 
// temporary file.  The temporary file name is generated
// when the user logs on, and is saved in all subsequent
// web pages as a hidden variable.  It could also be saved
// as a cookie.  The replay count is used to detect 
// replays of a previous HTML page.  It will be automatically
// updated before the temporary file is written.
//
///////////////////////////////////////////////////////////////

int	CLogisaExtension::saveTempfileData(char* filename, 
									   CString& user, 
									   CString& pw, 
									   CString& client, 
									   CString& sys, 
									   int replay,
									   const int tctualen,
									   const char* tctua)

{
	int		rc=0;		// return code variable
	FILE	*tempfile;	// file handle for the temporary file
	STATUS_FILE_HEADER	hdrData;	// Status file header data

	// open the file for output, erasing any previous files
	tempfile = fopen(filename, "wb");

	// Check if it worked
	if (tempfile == NULL)
	{
		// Didn't work, set return code to 1
		rc = 1;
	}
	else
	{
		// file is open, build the first part of the file
		// Get the user id, padded with spaces to 8 characters
		to_comm(user, hdrData.userName, sizeof(hdrData.userName), ALPHA_FIELD); 

		// Get the password, padded with spaces to 8 characters
		to_comm(pw, hdrData.passWord, sizeof(hdrData.passWord), ALPHA_FIELD); 

		// Get the system name, padded with spaces to 8 characters
		to_comm(sys, hdrData.sysName, sizeof(hdrData.sysName), ALPHA_FIELD); 

		// Get the client name, padded with spaces to 4 characters
		to_comm(client, hdrData.clientName, sizeof(hdrData.clientName), ALPHA_FIELD); 

		// Set the replay count to zero
		int_to_comm(replay, hdrData.replayCount, sizeof(hdrData.replayCount));

		// Set the TCT user area length field
		int_to_comm(tctualen, hdrData.tctuaLength, sizeof(hdrData.tctuaLength));

		// Set the COMMAREA length field
		int_to_comm(0, hdrData.caLength, sizeof(hdrData.caLength));

		// Write the system name to the file
		fwrite(&hdrData, 1, sizeof(hdrData), tempfile);

		// Check if there is a TCTUA to save
		if (tctualen > 0)
		{
			fwrite(tctua, 1, tctualen, tempfile);
		}

		// Close the file
		fclose(tempfile);
	}

	return rc;
}

void CLogisaExtension::procLogon(CHttpServerContext * pCtxt, 
								 LPTSTR cicsuser, 
								 LPTSTR cicspw, 
								 LPTSTR cicssys, 
								 LPTSTR cicsclient, 
								 LPTSTR submit)
{
	Cvartable vars;
    char   *filename;
	CString	user;
	CString	pw;
	CString	sys;
	CString	client;
	int    RCode;
	char   writerr[100];
	char   statusfile[MAX_DATA_LEN + 1];

    if (strcmp(submit, "Logon to CICS") == 0)
	{
		// Capture the input form variables
		vars.AddVar("CICSUSER", cicsuser);
		vars.AddVar("CICSPW", cicspw);
		vars.AddVar("CICSSYS", cicssys);
		vars.AddVar("CICSCLIENT", cicsclient);

	    filename = HTML_LOGON_FILE;

		// execute the CICS password verification transaction
		int rc = performLogon(vars, &RCode);

		// check for ECI errors
		if (rc == Ccl::noError)
		{
			// Check for a normal return code 
			if (RCode == 0)
			{
				filename = HTML_INQY_FILE;
				// Create a temporary status file
				// First, create a temporary file name
				generateFilename(statusfile, cicsclient, cicssys);

				// Next, create the actual file and save the data
				user = cicsuser;
				pw = cicspw; 
				client = cicsclient;
				sys = cicssys;

				// Save the initial transaction status
				// There is no TCTUA nor user COMMAREA, so pass zeros for the lengths
				// and NULLs for the addresses
				saveTempfileData(statusfile, user, pw, client, sys, 0, 0, NULL);

				// Save the file name and replay count in the variable table
				vars.AddVar("CICSSTATUSFILE", statusfile);
				vars.AddVar("REPLAYCOUNT", "00000000");
			}
			else
			{
				// Set a variable with the error message 
				vars.AddVar("ERRMSG", "Invalid Userid or Password");
			} // endif 
		}

		// write the HTML page
		rc = vars.write_html_page(pCtxt, filename);

		if (rc != 0)
		{
			*pCtxt << "Error writing page\n";
			sprintf(writerr, "Return code is %d", rc);
			*pCtxt << writerr; 
			EndContent(pCtxt);
		}
	}
	else
	{
		// write the HTTP headers
		StartContent(pCtxt);
		WriteTitle(pCtxt);

		*pCtxt << _T("Invalid button value\r\n");
		*pCtxt << " SUBMIT " << submit << _T("\r\n");
		*pCtxt << " CICSUSER " << cicsuser << _T("\r\n");
		*pCtxt << " CICSPW " << cicspw << _T("\r\n");
		*pCtxt << " CICSSYS " << cicssys << _T("\r\n");
		*pCtxt << " CICSCLIENT " << cicsclient << _T("\r\n");

		EndContent(pCtxt);
	}
}

BOOL CLogisaExtension::OnParseError(CHttpServerContext* pCtxt, int nCause) 
{
	// TODO: Add your specialized code here and/or call the base class
	// Trace point for debugging
	ISAPITRACE("Entering OnParseError %d\n", nCause);
		//StartContent(pCtxt);
		//WriteTitle(pCtxt);
		//*pCtxt << _T("Parse Error\r\n");
		//EndContent(pCtxt);
	
	return CHttpServer::OnParseError(pCtxt, nCause);
}

void CLogisaExtension::procTest(CHttpServerContext* pCtxt, LPTSTR submit)
{
	StartContent(pCtxt);
	WriteTitle(pCtxt);

	*pCtxt << _T("This is a test message\r\n");

	EndContent(pCtxt);
}

//////////////////////////////////////////////////////
//
// This routine is to be used only during development 
// of the extension, when it is being executed in a 
// debug environment.  This code is not thread safe
// and should not be used in a production environment.
// This code should only compile when compiled in the
// DEBUG configuration, and should not create any output
// when compiled in a production environment.  This should
// allow any tracing statements to be left in the program.
//
//////////////////////////////////////////////////////

void CLogisaExtension::writeTraceLine(const char* traceData)

{
#ifdef _DEBUG
	FILE	*tracefile;
	int		year;
	char	currtime[20];	// work variable for ASCII time
	char	currdate[20];	// work variable for ASCII date
	SYSTEMTIME	curtime;	// Work variable for current time

	// open the trace file for append
	tracefile = fopen(TRACE_FILE_NAME, "a");

	if (tracefile != NULL)
	{
		// Get the current time and date
		GetLocalTime(&curtime);

		// Format the date and and time
		year = curtime.wYear%100;
		sprintf(currdate, "%2.2d/%2.2d/%2.2d", year, curtime.wMonth, curtime.wDay);
		sprintf(currtime, "%2.2d:%2.2d:%2.2d.%3.3d", curtime.wHour, curtime.wMinute, 
				curtime.wSecond, curtime.wMilliseconds);

		fprintf(tracefile, "%s %s %s\n", currdate,currtime, traceData);
		fclose(tracefile);
	}
#endif
}

//////////////////////////////////////////////////////////
//
// Subroutine to turn a 4-bit number into a hex character
//
//////////////////////////////////////////////////////////

void data2hex(char *ch)

{
   if (((*ch) >= 0) && ((*ch) <= 9))
   {
      (*ch) += 48;  // turn into number 0-9 
   }
   else
   {
      (*ch) += 55;  // turn into letter A-F 
   } // endif 
}

//////////////////////////////////////////////////////
//
// This routine is to be used only during development 
// of the extension, when it is being executed in a 
// debug environment.  This code is not thread safe
// and should not be used in a production environment.
// This code should only compile when compiled in the
// DEBUG configuration, and should not create any output
// when compiled in a production environment.  This should
// allow any tracing statements to be left in the program.
//
//////////////////////////////////////////////////////

void CLogisaExtension::dumpArea(const char* traceData, const int len)

{
#ifdef _DEBUG
	char*	envvar;			// pointer to trace file name environment variable
	FILE	*tracefile;
	int		year;
	char	currtime[20];	// work variable for ASCII time
	char	currdate[20];	// work variable for ASCII date
	SYSTEMTIME	curtime;	// Work variable for current time
	char	hexStr[41];
	char	asciiStr[41];
	char	ch;
	int		i;
	int		j;
	int		k;
	int		buffer=0;
	int		remaining;
	int		offset;
	int		count;

	// keep track of how much we have to do 
	remaining = len;

	// check if the CICSISATRACE environment variable is set
	envvar = getenv("CICSISATRACE");

	// Is the trace file name set in the environment?
	if (envvar == NULL)
	{
		// Use the default file name
		envvar = TRACE_FILE_NAME;
	}

	// open the trace file for append
	tracefile = fopen(envvar, "a");

	if (tracefile != NULL)
	{
		// Get the current time and date
		GetLocalTime(&curtime);

		// Format the date and and time
		year = curtime.wYear%100;
		sprintf(currdate, "%2.2d/%2.2d/%2.2d", year, curtime.wMonth, curtime.wDay);
		sprintf(currtime, "%2.2d:%2.2d:%2.2d.%3.3d", curtime.wHour, curtime.wMinute, 
				curtime.wSecond, curtime.wMilliseconds);
		fprintf(tracefile, "%s %s Data at location %8.8X length %d\n", 
				currdate,currtime,traceData,len);

		while (remaining > 0)
		{
			if (remaining > 16)
			{
				count = 16;
			}
			else
			{
				count = remaining;
			} // endif 

			// initialize the output strings and buffer offset 
			offset = buffer;
			i = 0;    // initialize variable offset 
			j = 0;    // initialize variable offset 
			k = 0;    // initialize counter 

			while (k < count)
			{
				ch = (unsigned char) traceData[buffer] >> 4;
				data2hex(&ch);
				hexStr[i++] = ch;

				ch = (unsigned char) traceData[buffer] & 0x0F;
				data2hex(&ch);
				hexStr[i++] = ch;

				if ((traceData[buffer] > ' ') &&
					((unsigned char) traceData[buffer] <= 127))
				{
					asciiStr[j++] = traceData[buffer];
				}
				else
				{
					asciiStr[j++] = '.';
				} // endif 

				// Go on to the next character 
				remaining--;
				buffer++;
				k++;   // increment counter 

				// Insert a blank for every fourth character 
				if (k%4 == 0)
				{
					hexStr[i++] = ' ';
				} // endif 
			} // endwhile 

			hexStr[i]   = '\0'; // Terminate the string 
			asciiStr[j] = '\0'; // Terminate the string 

			if (count > 0)
			{
				fprintf(tracefile, "%.4X %-36s  %s\n",
						offset,hexStr,asciiStr);
			}
		} // endwhile 
		fclose(tracefile);
	}
#endif
}